/*
    Copyright (C) 2005-2012  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"

static GtkWidget *textview = NULL;
static GtkTextBuffer *buffer = NULL;
static gboolean is_visible = FALSE;

G_MODULE_EXPORT void
on_btnShowDebug_clicked(GtkWidget *caller, gpointer data)
{
    GtkWidget *windowDebug;

    windowDebug = nntpgrab_gui_base_get_widget("windowDebug");
    gtk_widget_show(windowDebug);
    is_visible = TRUE;

    nntpgrab_glue_set_emit_log_messages(glue, TRUE);
}

G_MODULE_EXPORT gboolean
on_windowDebug_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    gtk_widget_hide(widget);
    is_visible = FALSE;

    if (buffer) {
        gtk_text_buffer_set_text(buffer, "", -1);
    }

    nntpgrab_glue_set_emit_log_messages(glue, FALSE);

    return TRUE;
}

static void
on_log_message_cb(NntpgrabGlue *obj, const char *component, NGLogLevel log_level, const char *message, gpointer data)
{
    GtkTextIter iter;
    GtkTextMark *mark;

    char msg[1024];

#if defined(WIN32) || defined(__APPLE__)
    g_print("DEBUG: %s - %i - %s\n", component, log_level, message);
#endif

    if (log_level == NG_LOG_LEVEL_FATAL) {
        GtkWidget *window =nntpgrab_gui_base_get_widget("windowMain");

        memset(msg, 0, sizeof(msg));
        snprintf(msg, sizeof(msg) - 1, "%s - %s", component, message);

        g_print("%s - %s\n", component, message);

        nntpgrab_gui_base_dialog_show(window, msg, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);
        g_idle_add((GSourceFunc) gtk_main_quit, NULL);
    }

    if (log_level == NG_LOG_LEVEL_WARNING || log_level == NG_LOG_LEVEL_ERROR) {
        g_warning("%s - %s", component, message);
    }

    if (!is_visible) {
        return;
    }

    memset(msg, 0, sizeof(msg));
    snprintf(msg, sizeof(msg) - 1, "%s\t%i\t%s\n", component, log_level, message);
    gtk_text_buffer_get_end_iter(buffer, &iter);
    gtk_text_buffer_insert(buffer, &iter, msg, -1);

    /* Move the iterator to the beginning of line, so we don't scroll
     * in horizontal direction
     */
    gtk_text_iter_set_line_offset (&iter, 0);

    /* and place the mark at iter. the mark will stay there after we
     * insert some text at the end because it has right gravity.
     */
    mark = gtk_text_buffer_get_mark (buffer, "scroll");
    gtk_text_buffer_move_mark (buffer, mark, &iter);

    /* Scroll the mark onscreen.
     */
    gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW(textview), mark);
}

void
debug_initialize(void)
{
    GtkWidget *windowDebug;
    GtkTextIter iter;

    windowDebug = nntpgrab_gui_base_get_widget("windowDebug");

    textview = nntpgrab_gui_base_get_widget("textviewDebug");
    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));

    gtk_text_buffer_get_end_iter (buffer, &iter);
    gtk_text_buffer_create_mark (buffer, "scroll", &iter, TRUE);

    nntpgrab_glue_signal_connect(glue, "log_message", NG_CALLBACK(on_log_message_cb), windowDebug);
}

