#include "WidgetConfig.h"
#include "ui_WidgetConfig.h"

WidgetConfig::WidgetConfig(QNNTPGrabGlue *glue, WidgetConnections *widgetConnections, WidgetDownloadQueue *widgetDownloadQueue, MainWindow *mainwindow, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WidgetConfig)
{
    this->glue = glue;

    ui->setupUi(this);

    /* Create the individual tabs */
    servers = new WidgetConfigServers(glue);
    ui->horizontalLayout->addWidget(servers);

    folders = new WidgetConfigFolders(glue);
    folders->hide();
    ui->horizontalLayout->addWidget(folders);

    notifications = new WidgetConfigNotifications(glue, mainwindow);
    notifications->hide();
    ui->horizontalLayout->addWidget(notifications);

    par2 = new WidgetConfigPAR2(glue);
    par2->hide();
    ui->horizontalLayout->addWidget(par2);

    display = new WidgetConfigDisplay(widgetConnections, widgetDownloadQueue, mainwindow);
    display->hide();
    ui->horizontalLayout->addWidget(display);

#if 0
    nzbcreator = new WidgetConfigNZBCreator();
    nzbcreator->hide();
    ui->horizontalLayout->addWidget(nzbcreator);
#endif

    auto_import = new WidgetConfigAutoImport(mainwindow, glue);
    auto_import->hide();
    ui->horizontalLayout->addWidget(auto_import);

    plugins = new WidgetConfigPlugins(glue);
    plugins->hide();
    ui->horizontalLayout->addWidget(plugins);
}

WidgetConfig::~WidgetConfig()
{
    delete ui;
}

void WidgetConfig::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfig::on_listWidgetConfig_currentRowChanged(int currentRow)
{
    servers->hide();
    folders->hide();
    notifications->hide();
    par2->hide();
    display->hide();
#if 0
    nzbcreator->hide();
#endif
    auto_import->hide();
    plugins->hide();

    switch (currentRow) {
        case 0:
            servers->show();
            break;

        case 1:
            folders->show();
            break;

        case 2:
            notifications->show();
            break;

        case 3:
            par2->show();
            break;

        case 4:
            display->show();
            break;

#if 0
        case 5:
            nzbcreator->show();
            break;
#endif

        case 5:
            auto_import->show();
            break;

        case 6:
            plugins->show();
            break;

        default:
            break;
    }
}
