#include "WidgetConfigAutoImport.h"
#include "ui_WidgetConfigAutoImport.h"
#include "ConfigGuiOpts.h"

WidgetConfigAutoImport::WidgetConfigAutoImport(MainWindow *mainwindow, QNNTPGrabGlue *glue, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WidgetConfigAutoImport),
    auto_import(mainwindow, glue)
{
    ui->setupUi(this);

    this->glue = glue;

    model = new QDirModel();
    //model->setRootPath(QDir::rootPath());
    model->setReadOnly(true);
    model->setResolveSymlinks(true);
    model->setFilter(QDir::Dirs | QDir::AllDirs | QDir::Drives | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);

    ui->treeViewAutoImportLocal->setModel(model);
    ui->treeViewAutoImportLocal->setHeaderHidden(true);
    ui->treeViewAutoImportLocal->header()->hideSection(1);
    ui->treeViewAutoImportLocal->header()->hideSection(2);
    ui->treeViewAutoImportLocal->header()->hideSection(3);

    connect(ui->chkEnableAutoImportLocal, SIGNAL(toggled(bool)), SLOT(chkEnableAutoImportLocal_clicked(bool)));
    connect(ui->treeViewAutoImportLocal, SIGNAL(clicked(QModelIndex)), SLOT(treeViewAutoImportLocal_clicked(QModelIndex)));
    connect(ui->radMoveFileAfterImport, SIGNAL(toggled(bool)), SLOT(radMoveFileAfterImport_toggled(bool)));

    if (glue->getIsStandalone()) {
        ui->chkEnableAutoImportServer->hide();
        ui->groupBoxAutoImportServer->hide();
    } else {
        ConfigGuiOpts opts;

        ui->chkEnableAutoImportLocal->setChecked(opts.getEnableAutoImport());

        //QModelIndex idx = model->setRootPath(opts.getAutoImportDirectory());
        QModelIndex idx = model->index(opts.getAutoImportDirectory());
        ui->treeViewAutoImportLocal->scrollTo(idx);
        ui->treeViewAutoImportLocal->setCurrentIndex(idx);

        connect(ui->chkEnableAutoImportServer, SIGNAL(toggled(bool)), SLOT(chkEnableAutoImportServer_clicked(bool)));
        connect(ui->editAutoImportServer, SIGNAL(textChanged(QString)), SLOT(editAutoImportServer_textChanged(QString)));
        connect(ui->btnAutoImportServerApply, SIGNAL(clicked()), SLOT(btnAutoImportServerApply_clicked()));
    }

    configChanged();
    connect(glue, SIGNAL(configChanged()), SLOT(configChanged()));
}

WidgetConfigAutoImport::~WidgetConfigAutoImport()
{
    delete model;
    delete ui;
}

void WidgetConfigAutoImport::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfigAutoImport::configChanged()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    ui->radMoveFileAfterImport->setChecked(opts.move_file_after_auto_import);
    ui->radDeleteFileAfterImport->setChecked(!opts.move_file_after_auto_import);

    if (glue->getIsStandalone()) {
        /* Update the local configuration */
        if (ui->chkEnableAutoImportLocal->isChecked() != opts.enable_auto_import) {
            ui->chkEnableAutoImportLocal->setChecked(opts.enable_auto_import);
            if (opts.enable_auto_import) {
                //QModelIndex idx = model->setRootPath(opts.auto_import_directory);
                QModelIndex idx = model->index(opts.auto_import_directory);
                ui->treeViewAutoImportLocal->scrollTo(idx);
                ui->treeViewAutoImportLocal->setCurrentIndex(idx);
            }

#ifdef Q_OS_MAC
            /* Directory monitoring using GLib doesn't work on OSX
             * so we use Qt to perform directory monitoring */
            if (opts.enable_auto_import) {
                auto_import.enableWatch(opts.auto_import_directory);
            } else {
                auto_import.disableWatch();
            }
#endif
        }
    } else {
        /* Update the remote configuration */
        if (ui->chkEnableAutoImportServer->isChecked() != opts.enable_auto_import) {
            ui->chkEnableAutoImportServer->setChecked(opts.enable_auto_import);
            if (opts.enable_auto_import) {
                ui->editAutoImportServer->setText(opts.auto_import_directory);
                ui->btnAutoImportServerApply->setEnabled(false);
            }
        }
    }
}

void WidgetConfigAutoImport::chkEnableAutoImportLocal_clicked(bool toggled)
{
    if (glue->getIsStandalone()) {
        QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
        if (toggled != opts.enable_auto_import) {
            opts.enable_auto_import = toggled;
            glue->configSetOpts(opts);
        }
    } else {
        ConfigGuiOpts opts;
        if (toggled != opts.getEnableAutoImport()) {
            opts.setEnableAutoImport(toggled);

            if (toggled) {
                auto_import.enableWatch(opts.getAutoImportDirectory());
            } else {
                auto_import.disableWatch();
            }
        }
    }
}

void WidgetConfigAutoImport::treeViewAutoImportLocal_clicked(QModelIndex idx)
{
    QString path = model->filePath(idx);

    if (glue->getIsStandalone()) {
        QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
        if (opts.auto_import_directory != path) {
            opts.auto_import_directory = path;
            glue->configSetOpts(opts);
        }
    } else {
        ConfigGuiOpts opts;
        if (path != opts.getAutoImportDirectory()) {
            opts.setAutoImportDirectory(path);

            auto_import.enableWatch(opts.getAutoImportDirectory());
        }
    }
}

void WidgetConfigAutoImport::radMoveFileAfterImport_toggled(bool toggled)
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.move_file_after_auto_import != toggled) {
        opts.move_file_after_auto_import = toggled;
        glue->configSetOpts(opts);
    }
}

void WidgetConfigAutoImport::chkEnableAutoImportServer_clicked(bool toggled)
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (toggled != opts.enable_auto_import) {
        opts.enable_auto_import = toggled;
        glue->configSetOpts(opts);
    }
}

void WidgetConfigAutoImport::editAutoImportServer_textChanged(QString text)
{
    Q_UNUSED(text);
    ui->btnAutoImportServerApply->setEnabled(true);
}

void WidgetConfigAutoImport::btnAutoImportServerApply_clicked()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.auto_import_directory != ui->editAutoImportServer->text()) {
        opts.auto_import_directory = ui->editAutoImportServer->text();
        glue->configSetOpts(opts);
    }
}
