#include "WidgetConfigFolders.h"
#include "ui_WidgetConfigFolders.h"

WidgetConfigFolders::WidgetConfigFolders(QNNTPGrabGlue *glue, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WidgetConfigFolders)
{
    ui->setupUi(this);
    this->glue = glue;

    if (glue->getIsStandalone()) {
        /* We use QDirModel instead of QFileSystemModel because of QT-BUG #12934 */
        model = new QDirModel();
        //model->setRootPath(QDir::rootPath());
        model->setReadOnly(true);
        model->setResolveSymlinks(true);
        model->setFilter(QDir::Dirs | QDir::AllDirs | QDir::Drives | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);

        ui->treeViewDownloadFolder->setModel(model);
        ui->treeViewDownloadFolder->setHeaderHidden(true);
        ui->treeViewDownloadFolder->header()->hideSection(1);
        ui->treeViewDownloadFolder->header()->hideSection(2);
        ui->treeViewDownloadFolder->header()->hideSection(3);

        ui->treeViewTempFolder->setModel(model);
        ui->treeViewTempFolder->setHeaderHidden(true);
        ui->treeViewTempFolder->header()->hideSection(1);
        ui->treeViewTempFolder->header()->hideSection(2);
        ui->treeViewTempFolder->header()->hideSection(3);

        ui->groupBoxFoldersServer->hide();

        connect(ui->treeViewDownloadFolder, SIGNAL(clicked(QModelIndex)), SLOT(treeViewDownloadFolder_clicked(QModelIndex)));
        connect(ui->treeViewTempFolder, SIGNAL(clicked(QModelIndex)), SLOT(treeViewTempFolder_clicked(QModelIndex)));
    } else {
        model = NULL;
        ui->groupBoxDownloadFolder->hide();
        ui->groupBoxTempFolder->hide();

        connect(ui->editDownloadFolder, SIGNAL(textChanged(QString)), SLOT(editDownloadFolder_textChanged(QString)));
        connect(ui->editTempFolder, SIGNAL(textChanged(QString)), SLOT(editTempFolder_textChanged(QString)));
        connect(ui->btnChangeDownloadFolder, SIGNAL(clicked()), SLOT(btnChangeDownloadFolder_clicked()));
        connect(ui->btnChangeTempFolder, SIGNAL(clicked()), SLOT(btnChangeTempFolder_clicked()));
    }

    configChanged();
    connect(glue, SIGNAL(configChanged()), SLOT(configChanged()));
}

WidgetConfigFolders::~WidgetConfigFolders()
{
    if (model) {
        delete model;
    }

    delete ui;
}

void WidgetConfigFolders::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfigFolders::configChanged()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (glue->getIsStandalone()) {
        /* Set the tree views */
        //QModelIndex idx = model->setRootPath(opts.download_directory);
        QModelIndex idx = model->index(opts.download_directory);
        ui->treeViewDownloadFolder->scrollTo(idx);
        ui->treeViewDownloadFolder->setCurrentIndex(idx);

        //idx = model->setRootPath(opts.temp_directory);
        idx = model->index(opts.temp_directory);
        ui->treeViewTempFolder->scrollTo(idx);
        ui->treeViewTempFolder->setCurrentIndex(idx);
    } else {
        /* Set the text entry */
        ui->editDownloadFolder->setText(opts.download_directory);
        ui->editTempFolder->setText(opts.temp_directory);
        ui->btnChangeDownloadFolder->setEnabled(false);
        ui->btnChangeTempFolder->setEnabled(false);
    }
}

void WidgetConfigFolders::editDownloadFolder_textChanged(QString text)
{
    Q_UNUSED(text);
    ui->btnChangeDownloadFolder->setEnabled(true);
}

void WidgetConfigFolders::editTempFolder_textChanged(QString text)
{
    Q_UNUSED(text);
    ui->btnChangeTempFolder->setEnabled(true);
}

void WidgetConfigFolders::btnChangeDownloadFolder_clicked()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.download_directory != ui->editDownloadFolder->text()) {
        opts.download_directory = ui->editDownloadFolder->text();
        glue->configSetOpts(opts);
    }
}

void WidgetConfigFolders::btnChangeTempFolder_clicked()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.temp_directory != ui->editTempFolder->text()) {
        opts.temp_directory = ui->editTempFolder->text();
        glue->configSetOpts(opts);
    }
}

void WidgetConfigFolders::treeViewDownloadFolder_clicked(QModelIndex idx)
{
    QString path = model->filePath(idx);
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.download_directory != path) {
        opts.download_directory = path;
        glue->configSetOpts(opts);
    }
}

void WidgetConfigFolders::treeViewTempFolder_clicked(QModelIndex idx)
{
    QString path = model->filePath(idx);
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    if (opts.temp_directory != path) {
        opts.temp_directory = path;
        glue->configSetOpts(opts);
    }
}
